@@ -9,16 +9,15 @@
 #ifndef XSIMD_AVX512_INT32_HPP
 #define XSIMD_AVX512_INT32_HPP
 
-#include <cstdint>
-
-#include "xsimd_base.hpp"
+#include "xsimd_avx512_bool.hpp"
+#include "xsimd_avx512_int_base.hpp"
 
 namespace xsimd
 {
 
-    /**************************
+    /***************************
      * batch_bool<int32_t, 16> *
-     **************************/
+     ***************************/
 
     template <>
     struct simd_batch_traits<batch_bool<int32_t, 16>>
@@ -48,9 +47,9 @@ namespace xsimd
         };
     }
 
-    /*********************
+    /**********************
      * batch<int32_t, 16> *
-     *********************/
+     **********************/
 
     template <>
     struct simd_batch_traits<batch<int32_t, 16>>
@@ -62,71 +61,18 @@ namespace xsimd
     };
 
     template <>
-    class batch<int32_t, 16> : public simd_batch<batch<int32_t, 16>>
+    class batch<int32_t, 16> : public avx512_int_batch<int32_t, 16>
     {
     public:
 
-        using self_type = batch<int32_t, 16>;
-        using base_type = simd_batch<self_type>;
-
-        batch();
-        explicit batch(int32_t i);
-        batch(int32_t i0, int32_t i1,  int32_t i2,  int32_t i3,  int32_t i4,  int32_t i5,  int32_t i6,  int32_t i7,
-              int32_t i8, int32_t i9, int32_t i10, int32_t i11, int32_t i12, int32_t i13, int32_t i14, int32_t i15);
-        explicit batch(const int32_t* src);
-        batch(const int32_t* src, aligned_mode);
-        batch(const int32_t* src, unaligned_mode);
-        batch(const __m512i& rhs);
-        batch& operator=(const __m512i& rhs);
-
-        operator __m512i() const;
-
-        batch& load_aligned(const int32_t* src);
-        batch& load_unaligned(const int32_t* src);
-
-        batch& load_aligned(const int64_t* src);
-        batch& load_unaligned(const int64_t* src);
-
-        batch& load_aligned(const float* src);
-        batch& load_unaligned(const float* src);
-
-        batch& load_aligned(const double* src);
-        batch& load_unaligned(const double* src);
-
-        batch& load_aligned(const int8_t* src);
-        batch& load_unaligned(const int8_t* src);
-
-        batch& load_aligned(const uint8_t* src);
-        batch& load_unaligned(const uint8_t* src);
-
-        void store_aligned(int32_t* dst) const;
-        void store_unaligned(int32_t* dst) const;
-
-        void store_aligned(int64_t* dst) const;
-        void store_unaligned(int64_t* dst) const;
-
-        void store_aligned(float* dst) const;
-        void store_unaligned(float* dst) const;
-
-        void store_aligned(double* dst) const;
-        void store_unaligned(double* dst) const;
-
-        void store_aligned(int8_t* dst) const;
-        void store_unaligned(int8_t* dst) const;
-
-        void store_aligned(uint8_t* dst) const;
-        void store_unaligned(uint8_t* dst) const;
-
+        using base_type = avx512_int_batch<int32_t, 16>;
+        using base_type::base_type;
         using base_type::load_aligned;
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
 
-        int32_t operator[](std::size_t index) const;
-
-    private:
-
-        __m512i m_value;
+        XSIMD_DECLARE_LOAD_STORE_INT32(int32_t, 16);
     };
 
     batch<int32_t, 16> operator<<(const batch<int32_t, 16>& lhs, int32_t rhs);
@@ -134,105 +80,79 @@ namespace xsimd
     batch<int32_t, 16> operator<<(const batch<int32_t, 16>& lhs, const batch<int32_t, 16>& rhs);
     batch<int32_t, 16> operator>>(const batch<int32_t, 16>& lhs, const batch<int32_t, 16>& rhs);
 
-    /************************************
+    /*************************************
      * batch<int32_t, 16> implementation *
-     ************************************/
+     *************************************/
 
-    inline batch<int32_t, 16>::batch()
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const int8_t* src)
     {
-    }
-
-    inline batch<int32_t, 16>::batch(int32_t i)
-        : m_value(_mm512_set1_epi32(i))
-    {
-    }
-
-    inline batch<int32_t, 16>::batch(int32_t i0, int32_t i1,  int32_t i2,  int32_t i3,  int32_t i4,  int32_t i5,  int32_t i6,  int32_t i7,
-                                     int32_t i8, int32_t i9, int32_t i10, int32_t i11, int32_t i12, int32_t i13, int32_t i14, int32_t i15)
-        : m_value(_mm512_setr_epi32(i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11, i12, i13, i14, i15))
-    {
-    }
-
-    inline batch<int32_t, 16>::batch(const int32_t* src)
-        : m_value(_mm512_loadu_si512((__m512i const*)src))
-    {
-    }
-
-    inline batch<int32_t, 16>::batch(const int32_t* src, aligned_mode)
-        : m_value(_mm512_load_si512((__m512i const*)src))
-    {
-    }
-
-    inline batch<int32_t, 16>::batch(const int32_t* src, unaligned_mode)
-        : m_value(_mm512_loadu_si512((__m512i const*)src))
-    {
-    }
-
-    inline batch<int32_t, 16>::batch(const __m512i& rhs)
-        : m_value(rhs)
-    {
-    }
-
-    inline batch<int32_t, 16>& batch<int32_t, 16>::operator=(const __m512i& rhs)
-    {
-        m_value = rhs;
+        __m128i tmp = _mm_load_si128((const __m128i*)src);
+        this->m_value = _mm512_cvtepi8_epi32(tmp);
         return *this;
     }
 
-    inline batch<int32_t, 16>::operator __m512i() const
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const int8_t* src)
     {
-        return m_value;
-    }
-
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const int32_t* src)
-    {
-        m_value = _mm512_load_si512((__m512i const*)src);
+        __m128i tmp = _mm_loadu_si128((const __m128i*)src);
+        this->m_value = _mm512_cvtepi8_epi32(tmp);
         return *this;
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const int32_t* src)
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const uint8_t* src)
     {
-        m_value = _mm512_loadu_si512((__m512i const*)src);
+        __m128i tmp = _mm_load_si128((const __m128i*)src);
+        this->m_value = _mm512_cvtepu8_epi32(tmp);
         return *this;
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const int64_t* src)
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const uint8_t* src)
     {
-        alignas(64) int32_t tmp[16];
-        tmp[0] = static_cast<int32_t>(src[0]);
-        tmp[1] = static_cast<int32_t>(src[1]);
-        tmp[2] = static_cast<int32_t>(src[2]);
-        tmp[3] = static_cast<int32_t>(src[3]);
-        tmp[4] = static_cast<int32_t>(src[4]);
-        tmp[5] = static_cast<int32_t>(src[5]);
-        tmp[6] = static_cast<int32_t>(src[6]);
-        tmp[7] = static_cast<int32_t>(src[7]);
-        tmp[8] = static_cast<int32_t>(src[8]);
-        tmp[9] = static_cast<int32_t>(src[9]);
-        tmp[10] = static_cast<int32_t>(src[10]);
-        tmp[11] = static_cast<int32_t>(src[11]);
-        tmp[12] = static_cast<int32_t>(src[12]);
-        tmp[13] = static_cast<int32_t>(src[13]);
-        tmp[14] = static_cast<int32_t>(src[14]);
-        tmp[15] = static_cast<int32_t>(src[15]);
-        m_value = _mm512_load_si512((__m512i const*)tmp);
+        __m128i tmp = _mm_loadu_si128((const __m128i*)src);
+        this->m_value = _mm512_cvtepu8_epi32(tmp);
         return *this;
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const int64_t* src)
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const int16_t* src)
     {
-        return load_aligned(src);
+        __m256i tmp = _mm256_load_si256((const __m256i*)src);
+        this->m_value = _mm512_cvtepi16_epi32(tmp);
+        return *this;
     }
 
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const int16_t* src)
+    {
+        __m256i tmp = _mm256_loadu_si256((const __m256i*)src);
+        this->m_value = _mm512_cvtepi16_epi32(tmp);
+        return *this;
+    }
+
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const uint16_t* src)
+    {
+        __m256i tmp = _mm256_load_si256((const __m256i*)src);
+        this->m_value = _mm512_cvtepu16_epi32(tmp);
+        return *this;
+    }
+
+    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const uint16_t* src)
+    {
+        __m256i tmp = _mm256_loadu_si256((const __m256i*)src);
+        this->m_value = _mm512_cvtepu16_epi32(tmp);
+        return *this;
+    }
+
+    //XSIMD_DEFINE_LOAD_STORE(int32_t, 16, uint32_t, 64)
+    XSIMD_DEFINE_LOAD_STORE(int32_t, 16, int64_t, 64)
+    XSIMD_DEFINE_LOAD_STORE(int32_t, 16, uint64_t, 64)
+   
     inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const float* src)
     {
-        m_value = _mm512_cvtps_epi32(_mm512_load_ps(src));
+        this->m_value = _mm512_cvtps_epi32(_mm512_load_ps(src));
         return *this;
     }
 
     inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const float* src)
     {
-        m_value = _mm512_cvtps_epi32(_mm512_loadu_ps(src));
+        this->m_value = _mm512_cvtps_epi32(_mm512_loadu_ps(src));
         return *this;
     }
 
@@ -240,8 +160,8 @@ namespace xsimd
     {
         __m256i tmp1 = _mm512_cvtpd_epi32(_mm512_load_pd(src));
         __m256i tmp2 = _mm512_cvtpd_epi32(_mm512_load_pd(src + 8));
-        m_value = _mm512_castsi256_si512(tmp1);
-        m_value = _mm512_inserti32x8(m_value, tmp2, 1);
+        this->m_value = _mm512_castsi256_si512(tmp1);
+        this->m_value = _mm512_inserti32x8(this->m_value, tmp2, 1);
         return *this;
     }
 
@@ -249,133 +169,85 @@ namespace xsimd
     {
         __m256i tmp1 = _mm512_cvtpd_epi32(_mm512_loadu_pd(src));
         __m256i tmp2 = _mm512_cvtpd_epi32(_mm512_loadu_pd(src + 8));
-        m_value = _mm512_castsi256_si512(tmp1);
-        m_value = _mm512_inserti32x8(m_value, tmp2, 1);
+        this->m_value = _mm512_castsi256_si512(tmp1);
+        this->m_value = _mm512_inserti32x8(this->m_value, tmp2, 1);
         return *this;
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const int8_t* src)
+    inline void batch<int32_t, 16>::store_aligned(int8_t* dst) const
     {
-        __m128i tmp = _mm_load_si128((const __m128i*)src);
-        m_value = _mm512_cvtepi8_epi32(tmp);
-        return *this;
+        __m128i tmp = _mm512_cvtepi32_epi8(this->m_value);
+        _mm_store_si128((__m128i*)dst, tmp);
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const int8_t* src)
+    inline void batch<int32_t, 16>::store_unaligned(int8_t* dst) const
     {
-        __m128i tmp = _mm_loadu_si128((const __m128i*)src);
-        m_value = _mm512_cvtepi8_epi32(tmp);
-        return *this;
+        __m128i tmp = _mm512_cvtepi32_epi8(this->m_value);
+        _mm_storeu_si128((__m128i*)dst, tmp);
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_aligned(const uint8_t* src)
+    inline void batch<int32_t, 16>::store_aligned(uint8_t* dst) const
     {
-        __m128i tmp = _mm_load_si128((const __m128i*)src);
-        m_value = _mm512_cvtepu8_epi32(tmp);
-        return *this;
+        __m128i tmp = _mm512_cvtusepi32_epi8(this->m_value);
+        _mm_store_si128((__m128i*)dst, tmp);
     }
 
-    inline batch<int32_t, 16>& batch<int32_t, 16>::load_unaligned(const uint8_t* src)
+    inline void batch<int32_t, 16>::store_unaligned(uint8_t* dst) const
     {
-        __m128i tmp = _mm_loadu_si128((const __m128i*)src);
-        m_value = _mm512_cvtepu8_epi32(tmp);
-        return *this;
+        __m128i tmp = _mm512_cvtusepi32_epi8(this->m_value);
+        _mm_storeu_si128((__m128i*)dst, tmp);
     }
 
-    inline void batch<int32_t, 16>::store_aligned(int32_t* dst) const
+    inline void batch<int32_t, 16>::store_aligned(int16_t* dst) const
     {
-        _mm512_store_si512((__m512i*)dst, m_value);
+        __m256i tmp = _mm512_cvtepi32_epi16(this->m_value);
+        _mm256_store_si256((__m256i*)dst, tmp);
     }
 
-    inline void batch<int32_t, 16>::store_unaligned(int32_t* dst) const
+    inline void batch<int32_t, 16>::store_unaligned(int16_t* dst) const
     {
-        _mm512_storeu_si512((__m512i*)dst, m_value);
+        __m256i tmp = _mm512_cvtepi32_epi16(this->m_value);
+        _mm256_storeu_si256((__m256i*)dst, tmp);
     }
 
-    inline void batch<int32_t, 16>::store_aligned(int64_t* dst) const
+    inline void batch<int32_t, 16>::store_aligned(uint16_t* dst) const
     {
-        alignas(64) int32_t tmp[16];
-        store_aligned(tmp);
-        dst[0] = int64_t(tmp[0]);
-        dst[1] = int64_t(tmp[1]);
-        dst[2] = int64_t(tmp[2]);
-        dst[3] = int64_t(tmp[3]);
-        dst[4] = int64_t(tmp[4]);
-        dst[5] = int64_t(tmp[5]);
-        dst[6] = int64_t(tmp[6]);
-        dst[7] = int64_t(tmp[7]);
-        dst[8] = int64_t(tmp[8]);
-        dst[9] = int64_t(tmp[9]);
-        dst[10] = int64_t(tmp[10]);
-        dst[11] = int64_t(tmp[11]);
-        dst[12] = int64_t(tmp[12]);
-        dst[13] = int64_t(tmp[13]);
-        dst[14] = int64_t(tmp[14]);
-        dst[15] = int64_t(tmp[15]);
+        __m256i tmp = _mm512_cvtusepi32_epi16(this->m_value);
+        _mm256_store_si256((__m256i*)dst, tmp);
     }
 
-    inline void batch<int32_t, 16>::store_unaligned(int64_t* dst) const
+    inline void batch<int32_t, 16>::store_unaligned(uint16_t* dst) const
     {
-        store_aligned(dst);
+        __m256i tmp = _mm512_cvtusepi32_epi16(this->m_value);
+        _mm256_storeu_si256((__m256i*)dst, tmp);
     }
 
     inline void batch<int32_t, 16>::store_aligned(float* dst) const
     {
-        _mm512_store_ps(dst, _mm512_cvtepi32_ps(m_value));
+        _mm512_store_ps(dst, _mm512_cvtepi32_ps(this->m_value));
     }
 
     inline void batch<int32_t, 16>::store_unaligned(float* dst) const
     {
-        _mm512_storeu_ps(dst, _mm512_cvtepi32_ps(m_value));
+        _mm512_storeu_ps(dst, _mm512_cvtepi32_ps(this->m_value));
     }
 
     inline void batch<int32_t, 16>::store_aligned(double* dst) const
     {
-        __m256i tmp1 = _mm512_extracti32x8_epi32(m_value, 0);
-        __m256i tmp2 = _mm512_extracti32x8_epi32(m_value, 1);
+        __m256i tmp1 = _mm512_extracti32x8_epi32(this->m_value, 0);
+        __m256i tmp2 = _mm512_extracti32x8_epi32(this->m_value, 1);
         _mm512_store_pd(dst, _mm512_cvtepi32_pd(tmp1));
         _mm512_store_pd(dst + 8 , _mm512_cvtepi32_pd(tmp2));
     }
 
     inline void batch<int32_t, 16>::store_unaligned(double* dst) const
     {
-        __m256i tmp1 = _mm512_extracti32x8_epi32(m_value, 0);
-        __m256i tmp2 = _mm512_extracti32x8_epi32(m_value, 1);
+        __m256i tmp1 = _mm512_extracti32x8_epi32(this->m_value, 0);
+        __m256i tmp2 = _mm512_extracti32x8_epi32(this->m_value, 1);
         _mm512_store_pd(dst, _mm512_cvtepi32_pd(tmp1));
         _mm512_store_pd(dst + 8 , _mm512_cvtepi32_pd(tmp2));
     }
 
-    inline void batch<int32_t, 16>::store_aligned(int8_t* dst) const
-    {
-        __m128i tmp = _mm512_cvtepi32_epi8(m_value);
-        _mm_store_si128((__m128i*)dst, tmp);
-    }
-
-    inline void batch<int32_t, 16>::store_unaligned(int8_t* dst) const
-    {
-        __m128i tmp = _mm512_cvtepi32_epi8(m_value);
-        _mm_storeu_si128((__m128i*)dst, tmp);
-    }
-
-    inline void batch<int32_t, 16>::store_aligned(uint8_t* dst) const
-    {
-        __m128i tmp = _mm512_cvtusepi32_epi8(m_value);
-        _mm_store_si128((__m128i*)dst, tmp);
-    }
-
-    inline void batch<int32_t, 16>::store_unaligned(uint8_t* dst) const
-    {
-        __m128i tmp = _mm512_cvtusepi32_epi8(m_value);
-        _mm_storeu_si128((__m128i*)dst, tmp);
-    }
-
-    inline int32_t batch<int32_t, 16>::operator[](std::size_t index) const
-    {
-        alignas(64) int32_t x[16];
-        store_aligned(x);
-        return x[index & 15];
-    }
-
     namespace detail
     {
         template <>
