@@ -36,6 +36,8 @@ namespace xsimd
 
     public:
 
+        using base_type = simd_batch<batch<int8_t, 16>>;
+
         batch();
         explicit batch(int8_t d);
 
@@ -58,14 +60,21 @@ namespace xsimd
         batch& load_aligned(const int8_t* src);
         batch& load_unaligned(const int8_t* src);
 
+        batch& load_aligned(const uint8_t* src);
+        batch& load_unaligned(const uint8_t* src);
+
         void store_aligned(int8_t* dst) const;
         void store_unaligned(int8_t* dst) const;
 
-        batch& load_aligned(const char* src);
-        batch& load_unaligned(const char* src);
+        void store_aligned(uint8_t* dst) const;
+        void store_unaligned(uint8_t* dst) const;
 
-        void store_aligned(char* dst) const;
-        void store_unaligned(char* dst) const;
+        using base_type::load_aligned;
+        using base_type::load_unaligned;
+        using base_type::store_aligned;
+        using base_type::store_unaligned;
+
+        XSIMD_DECLARE_LOAD_STORE_INT8(int8_t, 16);
 
         int8_t operator[](std::size_t index) const;
 
@@ -148,14 +157,15 @@ namespace xsimd
         return load_aligned(src);
     }
 
-    inline batch<int8_t, 16>& batch<int8_t, 16>::load_aligned(const char* src)
+    inline batch<int8_t, 16>& batch<int8_t, 16>::load_aligned(const uint8_t* src)
     {
-        return load_aligned(reinterpret_cast<const int8_t*>(src));
+        m_value = vld1q_u8(src);
+        return *this;
     }
 
-    inline batch<int8_t, 16>& batch<int8_t, 16>::load_unaligned(const char* src)
+    inline batch<int8_t, 16>& batch<int8_t, 16>::load_unaligned(const uint8_t* src)
     {
-        return load_aligned(reinterpret_cast<const int8_t*>(src));
+        return load_aligned(src);
     }
 
     inline void batch<int8_t, 16>::store_aligned(int8_t* dst) const
@@ -168,16 +178,18 @@ namespace xsimd
         store_aligned(dst);
     }
 
-    inline void batch<int8_t, 16>::store_aligned(char* dst) const
+    inline void batch<int8_t, 16>::store_aligned(uint8_t* dst) const
     {
-        vst1q_s8(reinterpret_cast<int8_t*>(dst), m_value);
+        vst1q_u8(dst, m_value);
     }
 
-    inline void batch<int8_t, 16>::store_unaligned(char* dst) const
+    inline void batch<int8_t, 16>::store_unaligned(uint8_t* dst) const
     {
         store_aligned(dst);
     }
 
+    XSIMD_DEFINE_LOAD_STORE_INT8(int8_t, 16, 16)
+
     inline batch<int8_t, 16>::operator int8x16_t() const
     {
         return m_value;
@@ -399,4 +411,4 @@ namespace xsimd
 
 }
 
-#endif
\ No newline at end of file
+#endif
