@@ -17,9 +17,9 @@
 namespace xsimd
 {
 
-    /********************
-     * batch_bool<T, N> *
-     ********************/
+    /**************************
+     * batch_bool<int8_t, 16> *
+     **************************/
 
     template <>
     struct simd_batch_traits<batch_bool<int8_t, 16>>
@@ -68,9 +68,9 @@ namespace xsimd
         };
     }
 
-    /***********************
-     * sse_int_batch<T, N> *
-     ***********************/
+    /*********************
+     * batch<int8_t, 16> *
+     *********************/
 
     template <>
     struct simd_batch_traits<batch<int8_t, 16>>
@@ -95,7 +95,7 @@ namespace xsimd
     {
     public:
 
-        using base_class = sse_int_batch;
+        using base_class = sse_int_batch<int8_t, 16>;
         using base_class::base_class;
         using base_class::load_aligned;
         using base_class::load_unaligned;
@@ -119,32 +119,22 @@ namespace xsimd
         {
         }
 
-        batch& load_aligned(const char* src)
-        {
-            return load_aligned(reinterpret_cast<const int8_t*>(src));
-        }
-
-        batch& load_unaligned(const char* src)
-        {
-            return load_unaligned(reinterpret_cast<const int8_t*>(src));
-        }
-
-        void store_aligned(char* dst)
-        {
-            return store_aligned(reinterpret_cast<int8_t*>(dst));
-        }
-
-        void store_unaligned(char* dst)
-        {
-            return store_unaligned(reinterpret_cast<int8_t*>(dst));
-        }
+        XSIMD_DECLARE_LOAD_STORE_INT8(int8_t, 16);
     };
 
     template <>
     class batch<uint8_t, 16> : public sse_int_batch<uint8_t, 16>
     {
     public:
-        using sse_int_batch::sse_int_batch;
+
+        using base_class = sse_int_batch<uint8_t, 16>;
+        using base_class::base_class;
+        using base_class::load_aligned;
+        using base_class::load_unaligned;
+        using base_class::store_aligned;
+        using base_class::store_unaligned;
+
+        XSIMD_DECLARE_LOAD_STORE_INT8(uint8_t, 16);
     };
 
     batch<int8_t, 16> operator<<(const batch<int8_t, 16>& lhs, int32_t rhs);
@@ -152,6 +142,10 @@ namespace xsimd
     batch<uint8_t, 16> operator<<(const batch<uint8_t, 16>& lhs, int32_t rhs);
     batch<uint8_t, 16> operator>>(const batch<uint8_t, 16>& lhs, int32_t rhs);
 
+    /************************************
+     * batch<int8_t, 16> implementation *
+     ************************************/
+
     namespace detail
     {
         template <class T>
@@ -287,6 +281,8 @@ namespace xsimd
         };
     }
 
+    XSIMD_DEFINE_LOAD_STORE_INT8(int8_t, 16, 16)
+
     inline batch<int8_t, 16> operator<<(const batch<int8_t, 16>& lhs, int32_t rhs)
     {
         return sse_detail::shift_impl([](int8_t lhs, int32_t s) { return lhs << s; }, lhs, rhs);
@@ -297,6 +293,8 @@ namespace xsimd
         return sse_detail::shift_impl([](int8_t lhs, int32_t s) { return lhs >> s; }, lhs, rhs);
     }
 
+    XSIMD_DEFINE_LOAD_STORE_INT8(uint8_t, 16, 16)
+
     inline batch<uint8_t, 16> operator<<(const batch<uint8_t, 16>& lhs, int32_t rhs)
     {
         return sse_detail::shift_impl([](uint8_t lhs, int32_t s) { return lhs << s; }, lhs, rhs);
