@@ -20,6 +20,7 @@
 #include "xsimd_sse_double.hpp"
 #include "xsimd_sse_float.hpp"
 #include "xsimd_sse_int8.hpp"
+#include "xsimd_sse_int16.hpp"
 #include "xsimd_sse_int32.hpp"
 #include "xsimd_sse_int64.hpp"
 #include "xsimd_sse_complex.hpp"
@@ -30,6 +31,7 @@
 #include "xsimd_avx_double.hpp"
 #include "xsimd_avx_float.hpp"
 #include "xsimd_avx_int8.hpp"
+#include "xsimd_avx_int16.hpp"
 #include "xsimd_avx_int32.hpp"
 #include "xsimd_avx_int64.hpp"
 #include "xsimd_avx_complex.hpp"
